import { AsyncPipe, formatDate } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, Directive, Input, inject, Component, ViewEncapsulation, ChangeDetectionStrategy, Optional, Inject, ViewChild, ContentChildren, NgModule, LOCALE_ID } from '@angular/core';
import * as i1 from '@angular/material/form-field';
import { MAT_FORM_FIELD } from '@angular/material/form-field';
import { defer } from 'rxjs';
import { startWith, map, distinctUntilChanged } from 'rxjs/operators';

/** @docs-private */
function getNgxMatErrorDefMissingForError() {
    return Error(`'for' must be set for ngxMatErrorDef. See example: *ngxMatErrorDef="let error; for: 'pattern'`);
}

const NGX_MAT_ERROR_DEFAULT_OPTIONS = new InjectionToken('NGX_MAT_ERROR_DEFAULT_OPTIONS');
class NgxMatErrorDef {
    constructor(template) {
        this.template = template;
    }
    ngOnInit() {
        if (!this.ngxMatErrorDefFor) {
            throw getNgxMatErrorDefMissingForError();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrorDef, isStandalone: true, selector: "[ngxMatErrorDef]", inputs: { ngxMatErrorDefFor: "ngxMatErrorDefFor" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorDef, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxMatErrorDef]',
                    standalone: true,
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }]; }, propDecorators: { ngxMatErrorDefFor: [{
                type: Input
            }] } });
class NgxMatErrorOutlet {
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorOutlet, deps: [{ token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrorOutlet, isStandalone: true, selector: "[ngxMatErrorOutlet]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorOutlet, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngxMatErrorOutlet]',
                    standalone: true,
                }]
        }], ctorParameters: function () { return [{ type: i0.ViewContainerRef }]; } });
class NgxMatErrors {
    constructor(cdRef, matFormField) {
        this.cdRef = cdRef;
        this.matFormField = matFormField;
        this.messages = inject(NGX_MAT_ERROR_DEFAULT_OPTIONS);
        this.error$ = defer(() => {
            if (!this.control && this.matFormField) {
                this.control = this.matFormField._control;
            }
            if (!this.control) {
                return '';
            }
            const control = this.control.ngControl?.control;
            if (!control) {
                // TODO: Throw error;
                return '';
            }
            return this.initError(control);
        });
        this.messageKeys = new Set(Object.keys(this.messages));
    }
    initError(control) {
        return control.valueChanges.pipe(startWith(null), map(() => {
            this.errorOutlet.viewContainer.clear();
            if (!control.errors) {
                return '';
            }
            const errorKeys = Object.keys(control.errors);
            const customErrorMessage = this._getCustomErrorMessage(errorKeys);
            const errorOrErrorDef = customErrorMessage ??
                errorKeys.find((key) => this.messageKeys.has(key));
            if (!errorOrErrorDef) {
                return '';
            }
            const errors = control.errors;
            if (errorOrErrorDef instanceof NgxMatErrorDef) {
                this._populateErrorOutlet(errorOrErrorDef, errors);
                return '';
            }
            const message = this.messages[errorOrErrorDef];
            if (typeof message === 'function') {
                return message(errors[errorOrErrorDef]);
            }
            return message;
        }), distinctUntilChanged());
    }
    _populateErrorOutlet(errorDef, errors) {
        const context = {
            $implicit: errors[errorDef.ngxMatErrorDefFor],
        };
        this.errorOutlet.viewContainer.createEmbeddedView(errorDef.template, context);
        this.cdRef.markForCheck();
    }
    _getCustomErrorMessage(errors) {
        return this.customErrorMessages.find((customErrorMessage) => errors.some((error) => error === customErrorMessage.ngxMatErrorDefFor));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, deps: [{ token: i0.ChangeDetectorRef }, { token: MAT_FORM_FIELD, optional: true }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.4", type: NgxMatErrors, isStandalone: true, selector: "ngx-mat-errors, [ngx-mat-errors]", inputs: { control: ["ngx-mat-errors", "control"] }, host: { classAttribute: "ngx-mat-errors" }, queries: [{ propertyName: "customErrorMessages", predicate: NgxMatErrorDef, descendants: true }], viewQueries: [{ propertyName: "errorOutlet", first: true, predicate: NgxMatErrorOutlet, descendants: true, static: true }], ngImport: i0, template: `{{ error$ | async
    }}<ng-container ngxMatErrorOutlet></ng-container>`, isInline: true, dependencies: [{ kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "directive", type: NgxMatErrorOutlet, selector: "[ngxMatErrorOutlet]" }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrors, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngx-mat-errors, [ngx-mat-errors]',
                    template: `{{ error$ | async
    }}<ng-container ngxMatErrorOutlet></ng-container>`,
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    standalone: true,
                    imports: [AsyncPipe, NgxMatErrorOutlet],
                    host: {
                        class: 'ngx-mat-errors',
                    },
                }]
        }], ctorParameters: function () { return [{ type: i0.ChangeDetectorRef }, { type: i1.MatFormField, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [MAT_FORM_FIELD]
                }] }]; }, propDecorators: { control: [{
                type: Input,
                args: ['ngx-mat-errors']
            }], errorOutlet: [{
                type: ViewChild,
                args: [NgxMatErrorOutlet, { static: true }]
            }], customErrorMessages: [{
                type: ContentChildren,
                args: [NgxMatErrorDef, { descendants: true }]
            }] } });

class NgxMatErrorsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, imports: [NgxMatErrors, NgxMatErrorDef], exports: [NgxMatErrors, NgxMatErrorDef] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.4", ngImport: i0, type: NgxMatErrorsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [NgxMatErrors, NgxMatErrorDef],
                    exports: [NgxMatErrors, NgxMatErrorDef],
                }]
        }] });

function errorMessagesEnFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Please enter a value greater than or equal to ${error.min}.`,
        max: (error) => `Please enter a value less than or equal to ${error.max}.`,
        required: `This field is required.`,
        email: `Please enter a valid email address.`,
        minlength: (error) => `Please enter at least ${error.requiredLength} characters.`,
        maxlength: (error) => `Please enter no more than ${error.requiredLength} characters.`,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            return `Please enter a date greater than or equal to ${formatted ?? error.min}.`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            return `Please enter a date less than or equal to ${formatted ?? error.max}.`;
        },
    };
}
const NGX_MAT_ERROR_CONFIG_EN = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesEnFactory,
    deps: [LOCALE_ID],
};

function errorMessagesHuFactory(locale, format = 'shortDate') {
    return {
        min: (error) => `Nem lehet kisebb, mint ${error.min}.`,
        max: (error) => `Nem lehet nagyobb, mint ${error.max}.`,
        required: `Kötelező mező.`,
        email: `Nem érvényes e-mail cím.`,
        minlength: (error) => `Legalább ${error.requiredLength} karakter hosszú lehet.`,
        maxlength: (error) => `Legfeljebb ${error.requiredLength} karakter hosszú lehet.`,
        server: (error) => error,
        matDatepickerMin: (error) => {
            const formatted = formatDate(error.min, format, locale);
            // In Hungarian dates are ending with '.'
            return `Nem lehet korábbi dátum, mint ${formatted ?? error.min}`;
        },
        matDatepickerMax: (error) => {
            const formatted = formatDate(error.max, format, locale);
            // In Hungarian dates are ending with '.'
            return `Nem lehet későbbi dátum, mint ${formatted ?? error.max}`;
        },
    };
}
const NGX_MAT_ERROR_CONFIG_HU = {
    provide: NGX_MAT_ERROR_DEFAULT_OPTIONS,
    useFactory: errorMessagesHuFactory,
    deps: [LOCALE_ID]
};

/*
 * Public API Surface of ngx-mat-errors
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_MAT_ERROR_CONFIG_EN, NGX_MAT_ERROR_CONFIG_HU, NGX_MAT_ERROR_DEFAULT_OPTIONS, NgxMatErrorDef, NgxMatErrorOutlet, NgxMatErrors, NgxMatErrorsModule, errorMessagesEnFactory, errorMessagesHuFactory };
//# sourceMappingURL=ngx-mat-errors.mjs.map
